@@ -20,10 +20,11 @@ except ImportError:
     from mongoengine.errors import ValidationError
 from mongoengine.queryset import OperationError, Q
 from mongoengine.connection import get_db, DEFAULT_CONNECTION_NAME
+from mongoengine.base import NON_FIELD_ERRORS as MONGO_NON_FIELD_ERRORS
 from gridfs import GridFS
 
 from .documentoptions import DocumentMetaWrapper
-from .util import with_metaclass, load_field_generator
+from .util import with_metaclass, load_field_generator, format_mongo_validation_errors
 
 _fieldgenerator = load_field_generator()
 
@@ -35,44 +36,20 @@ def _get_unique_filename(name, db_alias=DEFAULT_CONNECTION_NAME, collection_name
         # file_ext includes the dot.
         name = os.path.join("%s_%s%s" % (file_root, next(count), file_ext))
     return name
-
-# The awesome Mongoengine ImageGridFsProxy wants to pull a field
-# from a document to get necessary data. Trouble is that this doesn't work
-# if the ImageField is stored on List or MapField. So we pass a nice fake
-# document to the proxy to get saving the file done. Yeah it really is that ugly.
-class FakeDocument(object):
-    _fields = {}
     
-    def __init__(self, key, field):
-        super(FakeDocument, self).__init__()
-        
-        self._fields[key] = field
-    
-    # We don't care if anything gets marked on this
-    # we do update a real field later though. That should
-    # trigger the same thing on the real document.
-    def _mark_as_changed(self, key):
-        pass
-    
-def _save_iterator_file(field, uploaded_file, file_data=None):
+def _save_iterator_file(field, instance, uploaded_file, file_data=None):
     """
     Takes care of saving a file for a list field. Returns a Mongoengine
     fileproxy object or the file field.
     """
-    fake_document = FakeDocument(field.name, field.field)
-    overwrote_instance = False
-    overwrote_key = False
     # for a new file we need a new proxy object
     if file_data is None:
-        file_data = field.field.proxy_class(db_alias=field.field.db_alias,
-                            collection_name=field.field.collection_name)
+        file_data = field.field.get_proxy_obj(key=field.name, instance=instance)
     
     if file_data.instance is None:
-        file_data.instance = fake_document
-        overwrote_instance = True
+        file_data.instance = instance
     if file_data.key is None:
         file_data.key = field.name
-        overwrote_key = True
     
     if file_data.grid_id:
         file_data.delete()
@@ -81,11 +58,6 @@ def _save_iterator_file(field, uploaded_file, file_data=None):
     filename = _get_unique_filename(uploaded_file.name, field.field.db_alias, field.field.collection_name)
     file_data.put(uploaded_file, content_type=uploaded_file.content_type, filename=filename)
     file_data.close()
-    
-    if overwrote_instance:
-        file_data.instance = None
-    if overwrote_key:
-        file_data.key = None
         
     return file_data
 
@@ -126,7 +98,7 @@ def construct_instance(form, instance, fields=None, exclude=None, ignore=None):
                 if uploaded_file is None:
                     continue
                 file_data = map_field.get(key, None)
-                map_field[key] = _save_iterator_file(f, uploaded_file, file_data)
+                map_field[key] = _save_iterator_file(f, instance, uploaded_file, file_data)
             setattr(instance, f.name, map_field)
         elif isinstance(f, ListField):
             list_field = getattr(instance, f.name)
@@ -138,7 +110,7 @@ def construct_instance(form, instance, fields=None, exclude=None, ignore=None):
                     file_data = list_field[i]
                 except IndexError:
                     file_data = None
-                file_obj = _save_iterator_file(f, uploaded_file, file_data)
+                file_obj = _save_iterator_file(f, instance, uploaded_file, file_data)
                 try:
                     list_field[i] = file_obj
                 except IndexError:
@@ -187,15 +159,9 @@ def save_instance(form, instance, fields=None, fail_message='saved',
     
     if commit and hasattr(instance, 'save'):
         # see BaseDocumentForm._post_clean for an explanation
-        if hasattr(form, '_delete_before_save'):
+        if len(form._meta._dont_save) > 0:
             data = instance._data
-            new_data = dict([(n, f) for n, f in data.items() if not n in form._delete_before_save])
-            if hasattr(instance, '_changed_fields'):
-                for field in form._delete_before_save:
-                    try:
-                        instance._changed_fields.remove(field)
-                    except ValueError:
-                        pass
+            new_data = dict([(n, f) for n, f in data.items() if not n in form._meta._dont_save])
             instance._data = new_data
             instance.save()
             instance._data = data
@@ -287,6 +253,8 @@ class ModelFormOptions(object):
         self.embedded_field = getattr(options, 'embedded_field_name', None)
         self.formfield_generator = getattr(options, 'formfield_generator', _fieldgenerator)
         
+        self._dont_save = []
+        
         
 class DocumentFormMetaclass(type):
     def __new__(cls, name, bases, attrs):
@@ -416,40 +384,47 @@ class BaseDocumentForm(BaseForm):
 
     def _post_clean(self):
         opts = self._meta
+        
         # Update the model instance with self.cleaned_data.
         self.instance = construct_instance(self, self.instance, opts.fields, opts.exclude)
+        changed_fields = getattr(self.instance, '_changed_fields', [])
 
         exclude = self._get_validation_exclusions()
-        # Clean the model instance's fields.
-        to_delete = []
         try:
             for f in self.instance._fields.values():
                 value = getattr(self.instance, f.name)
                 if f.name not in exclude:
                     f.validate(value)
-                elif value in EMPTY_VALUES:
+                elif value in EMPTY_VALUES and f.name not in changed_fields:
                     # mongoengine chokes on empty strings for fields
                     # that are not required. Clean them up here, though
                     # this is maybe not the right place :-)
-                    to_delete.append(f.name)
+                    opts._dont_save.append(f.name)
         except ValidationError as e:
             err = {f.name: [e.message]}
             self._update_errors(err)
-        
-        # Add to_delete list to instance. It is removed in save instance
-        # The reason for this is, that the field must be deleted from the 
-        # instance before the instance gets saved. The changed instance gets 
-        # cached and the removed field is then missing on subsequent edits.
-        # To avoid that it has to be added to the instance after the instance 
-        # has been saved. Kinda ugly.
-        self._delete_before_save = to_delete 
 
-        # Call the model instance's clean method.
-        if hasattr(self.instance, 'clean'):
-            try:
-                self.instance.clean()
-            except ValidationError as e:
-                self._update_errors({NON_FIELD_ERRORS: e.messages})
+        
+        # Call validate() on the document. Since mongoengine
+        # does not provide an argument to specify which fields
+        # should be excluded during validation, we replace
+        # instance._fields_ordered with a version that does
+        # not include excluded fields. The attribute gets
+        # restored after validation.
+        original_fields = self.instance._fields_ordered
+        self.instance._fields_ordered = tuple(
+                [f for f in original_fields if f not in exclude]
+        )
+        try:
+            self.instance.validate()
+        except ValidationError as e:
+            if MONGO_NON_FIELD_ERRORS in e.errors:
+                error = e.errors.get(MONGO_NON_FIELD_ERRORS)
+            else:
+                error = e.message
+            self._update_errors({NON_FIELD_ERRORS: [error, ]})
+        finally:
+            self.instance._fields_ordered = original_fields
 
         # Validate uniqueness if needed.
         if self._validate_unique:
@@ -500,8 +475,6 @@ class BaseDocumentForm(BaseForm):
         
         return errors
                 
-    
-
     def save(self, commit=True):
         """
         Saves this ``form``'s cleaned_data into model instance
@@ -523,6 +496,7 @@ class BaseDocumentForm(BaseForm):
         return obj
     save.alters_data = True
 
+
 class DocumentForm(with_metaclass(DocumentFormMetaclass, BaseDocumentForm)):
     pass
     
@@ -559,12 +533,14 @@ def documentform_factory(document, form=DocumentForm, fields=None, exclude=None,
 
 
 class EmbeddedDocumentForm(with_metaclass(DocumentFormMetaclass, BaseDocumentForm)):
-
-    def __init__(self, parent_document, instance=None, position=None, *args, **kwargs):
+    
+    def __init__(self, parent_document, data=None, files=None, position=None, *args, **kwargs):
         if self._meta.embedded_field is not None and not \
                 self._meta.embedded_field in parent_document._fields:
             raise FieldError("Parent document must have field %s" % self._meta.embedded_field)
         
+        instance = kwargs.pop('instance', None)
+        
         if isinstance(parent_document._fields.get(self._meta.embedded_field), ListField):
             # if we received a list position of the instance and no instance
             # load the instance from the parent document and proceed as normal
@@ -578,7 +554,7 @@ class EmbeddedDocumentForm(with_metaclass(DocumentFormMetaclass, BaseDocumentFor
                 emb_list = getattr(parent_document, self._meta.embedded_field)
                 position = next((i for i, obj in enumerate(emb_list) if obj == instance), None)
             
-        super(EmbeddedDocumentForm, self).__init__(instance=instance, *args, **kwargs)
+        super(EmbeddedDocumentForm, self).__init__(data=data, files=files, instance=instance, *args, **kwargs)
         self.parent_document = parent_document
         self.position = position
         
